==> fred-regal-0.4.0.tar.gz: fred-regal-0.4.0/regal/keyset.py <==
"""Keyset client for registry service."""
from datetime import datetime
from typing import Any, Dict, Optional, Sequence, Union

import fred_api.registry.keyset.service_keyset_grpc_pb2_grpc
import grpc
from fred_api.registry.keyset.keyset_history_types_pb2 import KeysetHistoryReply, KeysetHistoryRequest
from fred_api.registry.keyset.keyset_info_types_pb2 import (DnsKey as DnsKeyMessage, KeysetIdReply, KeysetIdRequest,
                                                            KeysetInfoReply, KeysetInfoRequest)
from fred_api.registry.keyset.keyset_state_history_types_pb2 import KeysetStateHistoryReply, KeysetStateHistoryRequest
from fred_api.registry.keyset.keyset_state_types_pb2 import KeysetStateReply, KeysetStateRequest
from pydantic import BaseModel, root_validator

from .common import ObjectEvents, RegistryDecoder, _set_sponsoring_registrar
from .contact import ContactId
from .exceptions import InvalidHistoryInterval, KeysetDoesNotExist
from .object import BaseObjectHistoryId, BaseObjectId, RegistryObjectClient


class KeysetId(BaseObjectId):
    """Keyset id."""


class KeysetHistoryId(BaseObjectHistoryId):
    """Keyset history id."""


class DnsKey(BaseModel):
    """Dns key info."""

    flags: int
    protocol: int
    alg: int
    key: str


class Keyset(BaseModel):
    """Keyset info."""

    keyset_id: KeysetId
    keyset_handle: str
    keyset_history_id: Optional[KeysetHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    dns_keys: Sequence[DnsKey] = []
    technical_contacts: Sequence[ContactId] = []
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None

    _set_sponsoring_registrar = root_validator(pre=True, allow_reuse=True)(_set_sponsoring_registrar)


class KeysetDecoder(RegistryDecoder):
    """Keyset service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(DnsKeyMessage, self._decode_dns_key)
        self.set_exception_decoder(KeysetIdReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetInfoReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetHistoryReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(KeysetStateReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetStateHistoryReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)

    def _decode_dns_key(self, value: DnsKeyMessage) -> DnsKey:
        """Decode DNS key."""
        return DnsKey(flags=value.flags, protocol=value.protocol, alg=value.alg, key=value.key)


class KeysetClient(RegistryObjectClient[KeysetId, KeysetHistoryId]):
    """Keyset service gRPC client."""

    service = 'Keyset'
    object_id = 'keyset_id'
    object_history_id = 'keyset_history_id'
    object_handle = 'keyset_handle'

    def __init__(self, netloc: str, credentials: grpc.ChannelCredentials = None) -> None:
        """Initialize keyset client instance.

        Args:
            netloc: Netloc of keyset service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=KeysetDecoder(),
            grpc_modules=[fred_api.registry.keyset.service_keyset_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_keyset_id(self, handle: str) -> KeysetId:
        """Get keyset id.

        Args:
            handle: Keyset handle.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
        """
        return KeysetId(await self.get_object_id(request=KeysetIdRequest(), method='get_keyset_id', handle=handle))

    async def get_keyset_info(self, keyset_id: KeysetId, history_id: KeysetHistoryId = None) -> Keyset:
        """Get keyset info.

        Args:
            keyset_id: Keyset ID.
            history_id: Keyset history ID.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
        """
        result = await self.get_object_info(
            request=KeysetInfoRequest(), method='get_keyset_info',
            object_id=keyset_id, history_id=history_id,
        )
        return Keyset(**result)

    async def get_keyset_state(self, keyset_id: KeysetId) -> Dict[str, bool]:
        """Get keyset state consisting of state flags and their boolean values.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.

        Args:
            keyset_id: Keyset ID.
        """
        return await self.get_object_state(request=KeysetStateRequest(), method='get_keyset_state', object_id=keyset_id)

    async def get_keyset_history(
        self, keyset_id: KeysetId, start: Union[KeysetHistoryId, datetime] = None,
        end: Union[KeysetHistoryId, datetime] = None,
    ) -> Dict[str, Any]:
        """Get keyset history timeline.

        Args:
            keyset_id: Keyset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=KeysetHistoryRequest(), method='get_keyset_history',
            object_id=keyset_id, start=start, end=end,
        )

    async def get_keyset_state_history(
        self, keyset_id: KeysetId, start: Union[KeysetHistoryId, datetime] = None,
        end: Union[KeysetHistoryId, datetime] = None,
    ) -> Dict[str, Any]:
        """Get keyset state history timeline.

        Args:
            keyset_id: Keyset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=KeysetStateHistoryRequest(), method='get_keyset_state_history',
            object_id=keyset_id, start=start, end=end,
        )

==> fred-regal-0.6.0.tar.gz: fred-regal-0.6.0/regal/keyset.py <==
"""Keyset client for registry service."""
from datetime import datetime
from typing import Any, Dict, Optional, Sequence, Union

import fred_api.registry.keyset.service_keyset_grpc_pb2_grpc
import grpc
from fred_api.registry.keyset.keyset_history_types_pb2 import KeysetHistoryReply, KeysetHistoryRequest
from fred_api.registry.keyset.keyset_info_types_pb2 import (
    DnsKey as DnsKeyMessage,
    KeysetIdReply,
    KeysetIdRequest,
    KeysetInfoReply,
    KeysetInfoRequest,
)
from fred_api.registry.keyset.keyset_state_history_types_pb2 import KeysetStateHistoryReply, KeysetStateHistoryRequest
from fred_api.registry.keyset.keyset_state_types_pb2 import KeysetStateReply, KeysetStateRequest
from pydantic import BaseModel, root_validator

from .common import ObjectEvents, RegistryDecoder, _set_sponsoring_registrar
from .contact import ContactId
from .exceptions import InvalidHistoryInterval, KeysetDoesNotExist
from .object import BaseObjectHistoryId, BaseObjectId, RegistryObjectClient


class KeysetId(BaseObjectId):
    """Keyset id."""


class KeysetHistoryId(BaseObjectHistoryId):
    """Keyset history id."""


class DnsKey(BaseModel):
    """Dns key info."""

    flags: int
    protocol: int
    alg: int
    key: str


class Keyset(BaseModel):
    """Keyset info."""

    keyset_id: KeysetId
    keyset_handle: str
    keyset_history_id: Optional[KeysetHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    dns_keys: Sequence[DnsKey] = []
    technical_contacts: Sequence[ContactId] = []
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None

    _set_sponsoring_registrar = root_validator(pre=True, allow_reuse=True)(_set_sponsoring_registrar)


class KeysetDecoder(RegistryDecoder):
    """Keyset service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(DnsKeyMessage, self._decode_dns_key)
        self.set_exception_decoder(KeysetIdReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetInfoReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetHistoryReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(KeysetStateReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetStateHistoryReply.Exception.KeysetDoesNotExist, KeysetDoesNotExist)
        self.set_exception_decoder(KeysetStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)

    def _decode_dns_key(self, value: DnsKeyMessage) -> DnsKey:
        """Decode DNS key."""
        return DnsKey(flags=value.flags, protocol=value.protocol, alg=value.alg, key=value.key)


class KeysetClient(RegistryObjectClient[KeysetId, KeysetHistoryId]):
    """Keyset service gRPC client."""

    service = "Keyset"
    object_id = "keyset_id"
    object_history_id = "keyset_history_id"
    object_handle = "keyset_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None) -> None:
        """Initialize keyset client instance.

        Args:
            netloc: Netloc of keyset service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=KeysetDecoder(),
            grpc_modules=[fred_api.registry.keyset.service_keyset_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_keyset_id(self, handle: str) -> KeysetId:
        """Get keyset id.

        Args:
            handle: Keyset handle.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
        """
        return KeysetId(await self.get_object_id(request=KeysetIdRequest(), method="get_keyset_id", handle=handle))

    async def get_keyset_info(self, keyset_id: KeysetId, history_id: Optional[KeysetHistoryId] = None) -> Keyset:
        """Get keyset info.

        Args:
            keyset_id: Keyset ID.
            history_id: Keyset history ID.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
        """
        result = await self.get_object_info(
            request=KeysetInfoRequest(),
            method="get_keyset_info",
            object_id=keyset_id,
            history_id=history_id,
        )
        return Keyset(**result)

    async def get_keyset_state(self, keyset_id: KeysetId) -> Dict[str, bool]:
        """Get keyset state consisting of state flags and their boolean values.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.

        Args:
            keyset_id: Keyset ID.
        """
        return await self.get_object_state(request=KeysetStateRequest(), method="get_keyset_state", object_id=keyset_id)

    async def get_keyset_history(
        self,
        keyset_id: KeysetId,
        start: Optional[Union[KeysetHistoryId, datetime]] = None,
        end: Optional[Union[KeysetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get keyset history timeline.

        Args:
            keyset_id: Keyset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=KeysetHistoryRequest(),
            method="get_keyset_history",
            object_id=keyset_id,
            start=start,
            end=end,
        )

    async def get_keyset_state_history(
        self,
        keyset_id: KeysetId,
        start: Optional[Union[KeysetHistoryId, datetime]] = None,
        end: Optional[Union[KeysetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get keyset state history timeline.

        Args:
            keyset_id: Keyset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            KeysetDoesNotExist: If keyset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=KeysetStateHistoryRequest(),
            method="get_keyset_state_history",
            object_id=keyset_id,
            start=start,
            end=end,
        )
